package com.jkcredit.invoice.service.nocar.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.jkcredit.invoice.mapper.customer.CustomerCarRecMapper;
import com.jkcredit.invoice.mapper.customer.CustomerRecMapper;
import com.jkcredit.invoice.mapper.waybill.NoCarWaybillMapper;
import com.jkcredit.invoice.model.entity.customer.CustomerCarRec;
import com.jkcredit.invoice.model.entity.customer.CustomerRec;
import com.jkcredit.invoice.model.entity.waybill.NoCarWayBill;
import com.jkcredit.invoice.service.customer.CustomerOperService;
import com.jkcredit.invoice.service.customer.CustomerService;
import com.jkcredit.invoice.service.lowerService.NoCarService;
import com.jkcredit.invoice.service.nocar.NoCarBillWayImportService;
import com.jkcredit.invoice.service.nocar.NoCarBillWayService;
import com.jkcredit.invoice.util.RespR;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
@Slf4j
@Service
public class NoCarBillWayServiceImpl extends ServiceImpl<NoCarWaybillMapper,NoCarWayBill> implements NoCarBillWayService{
    @Autowired
    NoCarWaybillMapper noCarWaybillMapper;

    @Autowired
    CustomerCarRecMapper customerCarRecMapper;

    @Autowired
    NoCarService noCarService;

    @Autowired
    CustomerRecMapper customerRecMapper;

    @Autowired
    CustomerOperService customerOperService;

    @Autowired
    CustomerService customerService;

    @Autowired
    NoCarBillWayImportService noCarWaybillMapperImprt;
    @Override
    public IPage<List<NoCarWayBill>> findByPageAndWayBill(Page page, NoCarWayBill noCarWayBill) {
        return noCarWaybillMapper.selectAllByPage(page,noCarWayBill);
    }

    @Override
    public IPage<List<NoCarWayBill>> findByPageAndWayBillException(Page page, NoCarWayBill noCarWayBill) {
        return noCarWaybillMapper.selectAllByPageException(page,noCarWayBill);
    }

    @Override
    public RespR batchBillWayStart(List<NoCarWayBill> noCarWayBills,String customerName) {
        log.info("开始平台指令上传NoCarBillWayServiceImpl.batchBillWayStart,上传数量"+noCarWayBills.size());
        if(!customerService.checkMoneyEnough(customerName)){
            return new RespR(false,"余额不足");
        }
        StringBuffer stringBuffer = new StringBuffer();
        //保存批次号信息
        String batchNumber = customerOperService.saveCustomerOper(customerName,1);
        noCarWayBills.parallelStream().forEach(noCarWayBill -> {
            //根据税号，查询备案企业
            CustomerRec customerRec = new CustomerRec();
            customerRec.setCustomerName(customerName);
            customerRec.setCompanyReferencenum(noCarWayBill.getTaxplayerCode());
            customerRec = customerRecMapper.selectByCustomerNameAndCompanyReference(customerRec);
            noCarWayBill.setCompanyName(customerRec.getCompanyName());
            noCarWayBill.setCustomerName(customerName);
            noCarWayBill.setBatchNum(batchNumber);
            noCarWayBill.setBillwayStatus(-1);
            noCarWaybillMapperImprt.insertImport(noCarWayBill);
            if(customerRec == null){
                noCarWaybillMapperImprt.updateBillway(-2,"企业税号未备案",noCarWayBill);
                stringBuffer.append("#").append(noCarWayBill.getBillNum()).append(":").append("企业税号未备案#");
                return;
            }
            //1.查询是否有车辆备案
            CustomerCarRec customerCarRec = customerCarRecMapper.selectByCarNum(noCarWayBill.getPlateColor());

            //2.无车车辆备案时进行车辆备案
            if(customerCarRec== null || customerCarRec.getRecStatus()!=1){
                List<CustomerCarRec> customerCarRecs = new ArrayList<>();
                customerCarRec = new CustomerCarRec();
                customerCarRec.setCompanyName(noCarWayBill.getCompanyName());
                customerCarRec.setCarNum(noCarWayBill.getPlateNum());
                customerCarRec.setCarColor(noCarWayBill.getPlateColor());
                customerCarRec.setCustomerName(customerName);
                customerCarRec.setBusinessType("2");
                noCarWayBill.setTitleType(2);
                customerCarRecs.add(customerCarRec);
                RespR respR = noCarService.customerCarRec(customerCarRecs);
                if(respR.getCode() ==1){
                    stringBuffer.append("#").append(noCarWayBill.getBillNum()).append(":").append(respR.getMsg()).append("#");
                    noCarWaybillMapperImprt.updateBillway(-2,respR.getMsg(),noCarWayBill);
                    return;
                }
            }

            //3.进行上传
            RespR respR = noCarService.noCarWaybillStart(noCarWayBill);
            if (respR.getCode() == 1){
                noCarWaybillMapperImprt.updateBillway(-2,respR.getMsg(),noCarWayBill);
                stringBuffer.append("#").append(noCarWayBill.getBillNum()).append(":").append(respR.getMsg()).append("#");
            }
            noCarWaybillMapperImprt.updateBillway(1,"",noCarWayBill);
        });
        log.error("开始平台指令上传失败NoCarBillWayServiceImpl.batchBillWayStart:{}",stringBuffer.toString());
        if(!StringUtils.isEmpty(stringBuffer.toString())){
            return new RespR(false,stringBuffer.toString());
        }
        return new RespR(Boolean.TRUE);
    }

    @Override
    public RespR batchHsitoryBillWay(List<NoCarWayBill> noCarWayBills, String customerName){
        if(!customerService.checkMoneyEnough(customerName)){
            return new RespR(false,"余额不足");
        }
        StringBuffer stringBuffer = new StringBuffer();
        //保存批次号信息
        String batchNumber = customerOperService.saveCustomerOper(customerName,3);
        noCarWayBills.parallelStream().forEach(noCarWayBill -> {
            try {
                log.info("kai shi chu li li si yun dan"+noCarWayBill.getBillNum());
                //根据税号，查询备案企业
                CustomerRec customerRec = new CustomerRec();
                customerRec.setCustomerName(customerName);
                customerRec.setCompanyReferencenum(noCarWayBill.getTaxplayerCode());
                customerRec = customerRecMapper.selectByCustomerNameAndCompanyReference(customerRec);
                noCarWayBill.setCompanyName(customerRec.getCompanyName());
                noCarWayBill.setCustomerName(customerName);
                noCarWayBill.setBatchNum(batchNumber);
                noCarWayBill.setTitleType(2);
                noCarWayBill.setBillwayStatus(-1);
                noCarWaybillMapperImprt.insertImport(noCarWayBill);
                if(customerRec == null || customerRec.getRecStatus()!=1){
                    stringBuffer.append("#").append(noCarWayBill.getBillNum()).append(":").append("企业税号未备案#");
                    noCarWaybillMapperImprt.updateBillway(-2,"企业税号未备案#",noCarWayBill);
                    return;
                }

                //1.查询是否有车辆备案
                CustomerCarRec customerCarRec = customerCarRecMapper.selectByCarNum(noCarWayBill.getPlateNum());

                //2.无车车辆备案时进行车辆备案
                if(customerCarRec== null || customerCarRec.getRecStatus()!=1){
                    List<CustomerCarRec> customerCarRecs = new ArrayList<>();
                    customerCarRec = new CustomerCarRec();
                    customerCarRec.setCompanyName(noCarWayBill.getCompanyName());
                    customerCarRec.setCarNum(noCarWayBill.getPlateNum());
                    customerCarRec.setCarColor(noCarWayBill.getPlateColor());
                    customerCarRec.setCustomerName(customerName);
                    customerCarRec.setBusinessType("2");
                    customerCarRecs.add(customerCarRec);
                    RespR respR = noCarService.customerCarRec(customerCarRecs);
                    if(respR.getCode() ==1){
                        noCarWaybillMapperImprt.updateBillway(-2,respR.getMsg(),noCarWayBill);
                        stringBuffer.append("#").append(noCarWayBill.getBillNum()).append(":").append(respR.getMsg()).append("#");
                        return;
                    }
                }

                //3.kais指令上传
                NoCarWayBill noCarWayBillClo = null;
                try{
                    noCarWayBillClo = noCarWayBill.clone();
                }catch (Exception e){
                    e.printStackTrace();
                    noCarWayBillClo = noCarWayBill;
                }

                RespR respR = noCarService.noCarHisWaybillStart(noCarWayBill);
                if (respR.getCode() == 1){
                    stringBuffer.append("#").append(noCarWayBill.getBillNum()).append(":").append(respR.getMsg()).append("#");
                    noCarWaybillMapperImprt.updateBillway(-2,respR.getMsg(),noCarWayBill);
                    log.info(noCarWayBill.getBillNum()+"#开始指令失败"+respR.getMsg());
                }else{
                    log.info(noCarWayBill.getBillNum()+"#开始指令成功"+respR.getMsg());
                    //3.kais指令上传
                    respR = noCarService.noCarHisWaybillEnd(noCarWayBillClo);

                    if(respR.getCode() == 1){
                        noCarWaybillMapperImprt.updateBillway(1,respR.getMsg(),noCarWayBill);
                        log.info(noCarWayBill.getBillNum()+"#结束指令失败"+respR.getMsg());
                        stringBuffer.append("#").append(noCarWayBill.getBillNum()).append(":").append(respR.getMsg()).append("#");
                    }else{
                        noCarWaybillMapperImprt.updateBillway(2,respR.getMsg(),noCarWayBill);
                        log.info(noCarWayBill.getBillNum()+"#结束指令成功"+respR.getMsg());
                    }
                }
                log.info("kai shi chu li li si yun dan"+noCarWayBill.getBillNum());
            }catch (Exception e){
                e.printStackTrace();
            }


        });
        if(!StringUtils.isEmpty(stringBuffer.toString())){
            return new RespR(false,stringBuffer.toString());
        }
        return new RespR(Boolean.TRUE);
    }

    @Override
    public RespR batchBillWayEnd(List<NoCarWayBill> noCarWayBills, String customerName) {
        log.error("结束平台指令上传数量NoCarBillWayServiceImpl.batchBillWayEnd:{}",noCarWayBills.size());
        if(!customerService.checkMoneyEnough(customerName)){
            return new RespR(false,"余额不足");
        }
        StringBuffer stringBuffer = new StringBuffer();
        //保存批次号信息
        String batchNumber = customerOperService.saveCustomerOper(customerName,2);

        noCarWayBills.parallelStream().forEach(noCarWayBill -> {
            try {
                noCarWayBill.setBatchNumEnd(batchNumber);
                noCarWaybillMapperImprt.insertImport(noCarWayBill);
                RespR respR = noCarService.noCarWaybillEnd(noCarWayBill);
                if (respR.getCode() == 1){
                    noCarWaybillMapperImprt.updateBillway(-2,respR.getMsg(),noCarWayBill);
                    stringBuffer.append("#").append(noCarWayBill.getBillNum()).append(":").append(respR.getMsg()).append("#");
                }else{
                    noCarWaybillMapperImprt.updateBillway(2,respR.getMsg(),noCarWayBill);
                }
            }catch (Exception e){
                e.printStackTrace();
            }

        });

        if(!StringUtils.isEmpty(stringBuffer.toString())){
            log.error("开始平台指令上传失败NoCarBillWayServiceImpl.batchBillWayEnd:{}",stringBuffer.toString());
            return new RespR(false,stringBuffer.toString());
        }
        return new RespR(Boolean.TRUE);
    }
}
